#include "TorusGenerator.h"

GraphicalObject * TorusGenerator::CreateTorus (RawObject * raw_object, double r_t, double r_p, int rings, int slices)
{
	return CreateRings (raw_object, r_t, r_p, 0.0, 2 * MY_PI, rings, slices);
}

GraphicalObject * TorusGenerator::CreateRings (RawObject * raw_object, double r_t, double r_p, double min_t, double max_t, int rings, int slices)
{
	if (rings == 1)
	{
		return CreateRingPart (raw_object, r_t, r_p, min_t, 0.0, 2 * MY_PI, slices);
	}
	else
	{
		GroupObject * group = new GroupObject ();
		GraphicalObject * child_1; 
		GraphicalObject * child_2;
		int first_group_count;

		first_group_count = rings / 2;
		
		// split the finally generated objects into two groups
		child_1 = CreateRings (raw_object, r_t, r_p, min_t, (max_t - min_t) * first_group_count / rings + min_t, first_group_count, slices);
		child_2 = CreateRings (raw_object, r_t, r_p, min_t + (max_t - min_t) * first_group_count / rings, max_t, rings - first_group_count, slices);
		
		group->AddChild (child_1);
		group->AddChild (child_2);

		return  group;
	}
}

GraphicalObject * TorusGenerator::CreateRingPart (RawObject * raw_object, double r_t, double r_p, double curr_t, double min_p, double max_p, int slices)
{
	if (slices == 1)
	{
		TerminalObject * terminal;

		terminal = new TerminalObject (raw_object);

		float x, y, z;

		x = (r_t + r_p * cos (min_p)) * cos (curr_t);
		y = (r_t + r_p * cos (min_p)) * sin (curr_t);
		z = r_p * sin (min_p);

		terminal->position->SetConstantPosition (x, y, z, 0.0f, 0.0f, 0.0f);

		return terminal;
	}
	else
	{
		GroupObject * group = new GroupObject ();
		GraphicalObject * child_1;
		GraphicalObject * child_2;
		int first_group_count;

		first_group_count = slices / 2;
		child_1 = CreateRingPart (raw_object, r_t, r_p, curr_t, min_p, min_p + (max_p - min_p) * first_group_count / slices, first_group_count);
		child_2 = CreateRingPart (raw_object, r_t, r_p, curr_t, min_p + (max_p - min_p) * first_group_count / slices, max_p, slices - first_group_count);

		group->AddChild (child_1);
		group->AddChild (child_2);

		return group;
	}
}

GraphicalObject * TorusGenerator::CreateTorus (DLODObject * dlod_object, double r_t, double r_p, int rings, int slices)
{
	return CreateRings (dlod_object, r_t, r_p, 0.0, 2 * MY_PI, rings, slices);
}

GraphicalObject * TorusGenerator::CreateRings (DLODObject * dlod_object, double r_t, double r_p, double min_t, double max_t, int rings, int slices)
{
	if (rings == 1)
	{
		return CreateRingPart (dlod_object, r_t, r_p, min_t, 0.0, 2 * MY_PI, slices);
	}
	else
	{
		GroupObject * group = new GroupObject ();
		GraphicalObject * child_1; 
		GraphicalObject * child_2;
		int first_group_count;

		first_group_count = rings / 2;
		
		child_1 = CreateRings (dlod_object, r_t, r_p, min_t, (max_t - min_t) * first_group_count / rings + min_t, first_group_count, slices);
		child_2 = CreateRings (dlod_object, r_t, r_p, min_t + (max_t - min_t) * first_group_count / rings, max_t, rings - first_group_count, slices);

		group->AddChild (child_1);
		group->AddChild (child_2);

		return  group;
	}
}

GraphicalObject * TorusGenerator::CreateRingPart (DLODObject * dlod_object, double r_t, double r_p, double curr_t, double min_p, double max_p, int slices)
{
	if (slices == 1)
	{
		DLODObject * terminal;

		terminal = dlod_object->GetClone ();

		float x, y, z;

		x = (r_t + r_p * cos (min_p)) * cos (curr_t);
		y = (r_t + r_p * cos (min_p)) * sin (curr_t);
		z = r_p * sin (min_p);

		terminal->position->SetConstantPosition (x, y, z, 0.0f, 0.0f, 0.0f);

		return terminal;
	}
	else
	{
		GroupObject * group = new GroupObject ();
		GraphicalObject * child_1;
		GraphicalObject * child_2;
		int first_group_count;

		first_group_count = slices / 2;
		child_1 = CreateRingPart (dlod_object, r_t, r_p, curr_t, min_p, min_p + (max_p - min_p) * first_group_count / slices, first_group_count);
		child_2 = CreateRingPart (dlod_object, r_t, r_p, curr_t, min_p + (max_p - min_p) * first_group_count / slices, max_p, slices - first_group_count);

		group->AddChild (child_1);
		group->AddChild (child_2);

		return group;
	}
}

bool TorusGenerator::HasMore ()
{
	if (ring_index >= rings)
	{return false;}
	else
	{return true;}
}

void TorusGenerator::StartTorus (double r_t, double r_p, int rings, int slices)
{
	TorusGenerator::r_t = r_t;
	TorusGenerator::r_p = r_p;
	TorusGenerator::rings = rings;
	TorusGenerator::slices = slices;
	TorusGenerator::ring_index = 0;
	TorusGenerator::slice_index = 0;
}

double TorusGenerator::r_t = 0.0;
double TorusGenerator::r_p = 0.0;
int TorusGenerator::rings = 0;
int TorusGenerator::slices = 0;
int TorusGenerator::ring_index = 0;
int TorusGenerator::slice_index = 0;

Point TorusGenerator::GetNextPoint ()
{
	Point current_point;
	double t, p, r_t, r_p;

	r_t = TorusGenerator::r_t;
	r_p = TorusGenerator::r_p;
	t = 2.0 * MY_PI * TorusGenerator::ring_index / TorusGenerator::rings;
	p = 2.0 * MY_PI * TorusGenerator::slice_index / TorusGenerator::slices;

	current_point.x = (r_t + r_p * cos (p)) * cos (t);
	current_point.y = (r_t + r_p * cos (p)) * sin (t);
	current_point.z =  r_p * sin (p);

	TorusGenerator::slice_index += 1;
	if (TorusGenerator::slice_index == TorusGenerator::slices)
	{
		TorusGenerator::slice_index = 0;
		TorusGenerator::ring_index += 1;
	}

	return current_point;
}