#ifndef UTILITY_
#define UTILITY_

#include <stdio.h>
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "Geometry.h"

/**
 * \brief Some utility routines.
 *
 * Some helpful utility routines.
 */
class Utility
{
public:
	/**
	 * Draws a quad defined by four points.
	 *
	 * \param p1							First point of the quad.
	 * \param p2							Second point of the quad.
	 * \param p3							Third point of the quad.
	 * \param p4							Fourth point of the quad.
	 */
	static void DrawQuad (const Point& p1, const Point& p2, const Point& p3, const Point& p4);

	/**
	 * Draws a bounding box.
	 *
	 * \param bounding_box					The bounding box which shall be drawn.
	 */
	static void DrawBoundingBox (BoundingBox bounding_box);

	/**
	 * Sets the color used for drawing the quad or the bounding box.
	 *
	 * \param red							The red part of the drawing color.
	 * \param green							The green part of the drawing color.
	 * \param blue							The blue part of the drawing color.
	 * \param alpha							The alpha value of the drawing color.
	 */
	static void SetColor (float red, float green, float blue, float alpha);

	/**
	 * Retrieves a corner of the bounding box. The point is specified by an index in the range from 0 to 7.
	 *
	 * \param box							The bounding box for which a specific point shall be retrieved.
	 * \param index							Index specifying the point of the box.
	 * \return								A corner of the bounding box.
	 */
	static Point GetPointFromBox (const BoundingBox & box, int index);

private:
	/** Color used for drawing the quad and the bounding box. */
	static float color[4];
};

#endif